@@ -45,6 +45,7 @@ Currently supported backends:
 # pyrates internal _imports
 from ..computegraph import ComputeVar
 from .base_funcs import base_funcs
+from .. import PyRatesException
 
 # external _imports
 from typing import Optional, Dict, List, Union, Tuple, Callable, Iterable
@@ -135,6 +136,10 @@ class BaseBackend(CodeGen):
                 if imp not in self._imports:
                     self._imports.append(imp)
 
+        # public attributes
+        self.add_hist_arg = kwargs.pop('add_hist_arg', True)
+        self.lags = {}
+
         # private attributes
         self._float_precision = kwargs.pop('float_precision', 'float32')
         self._int_precision = kwargs.pop('int_precision', 'int32')
@@ -150,7 +155,7 @@ class BaseBackend(CodeGen):
             sys.path.append(fdir)
         else:
             sys.path.append(os.getcwd())
-        self._fdir = fdir
+        self.fdir = fdir
         self._fname = fname
         self._fend = kwargs.pop('file_ending', '.py')
 
@@ -217,6 +222,11 @@ class BaseBackend(CodeGen):
             lhs = f"{lhs}{idx}"
         self.add_code_line(f"{lhs} = {rhs}")
 
+    def add_var_hist(self, lhs: str, delay: Union[ComputeVar, float], state_idx: str, **kwargs):
+        idx = self._process_idx(state_idx)
+        d = self._process_delay(delay)
+        self.add_code_line(f"{lhs} = hist(t-{d})[{idx}]")
+
     def create_index_str(self, idx: Union[str, int, tuple], separator: str = ',', apply: bool = True,
                          **kwargs) -> Tuple[str, dict]:
 
@@ -250,7 +260,8 @@ class BaseBackend(CodeGen):
             *path, file = f.split('/')
             return '/'.join(path), file
 
-    def generate_func_head(self, func_name: str, state_var: str = 'y', return_var: str = 'dy', func_args: list = None):
+    def generate_func_head(self, func_name: str, state_var: str = 'y', return_var: str = 'dy', func_args: list = None,
+                           add_hist_func: bool = False):
 
         imports = self._imports
         helper_funcs = self._helper_funcs
@@ -259,6 +270,8 @@ class BaseBackend(CodeGen):
         else:
             func_args = []
         state_vars = ['t', state_var]
+        if add_hist_func:
+            state_vars.append('hist')
         _, indices = np.unique(func_args, return_index=True)
         func_args = state_vars + [func_args[idx] for idx in np.sort(indices)]
 
@@ -296,7 +309,7 @@ class BaseBackend(CodeGen):
         if to_file:
 
             # save rhs function to file
-            file = f'{self._fdir}/{self._fname}' if self._fdir else self._fname
+            file = f'{self.fdir}/{self._fname}' if self.fdir else self._fname
             with open(f'{file}{self._fend}', 'w') as f:
                 f.writelines(func_str)
                 f.close()
@@ -348,8 +361,8 @@ class BaseBackend(CodeGen):
         super().clear()
 
         # remove files and directories that have been created during simulation process
-        if self._fdir:
-            rmtree(self._fdir)
+        if self.fdir:
+            rmtree(self.fdir)
         else:
             try:
                 os.remove(f"{self._fname}{self._fend}")
@@ -360,6 +373,10 @@ class BaseBackend(CodeGen):
         if self._fname in sys.modules:
             del sys.modules[self._fname]
 
+    @staticmethod
+    def to_file(fn: str, **kwargs):
+        np.savez(fn, **kwargs)
+
     @staticmethod
     def register_vars(variables: list):
         pass
@@ -372,6 +389,10 @@ class BaseBackend(CodeGen):
     def expr_to_str(expr: str, args: tuple):
         return expr
 
+    @staticmethod
+    def get_hist_func(y: np.ndarray):
+        return lambda t: y
+
     def _get_func_info(self, name: str, **kwargs):
         return self._funcs[name]
 
@@ -388,6 +409,9 @@ class BaseBackend(CodeGen):
         except (TypeError, ValueError):
             return idx
 
+    def _process_delay(self, delay: Union[ComputeVar, float]) -> str:
+        return f"{delay}[{self._start_idx}]" if type(delay) is ComputeVar and delay.shape else f"{delay}"
+
     def _solve(self, solver: str, func: Callable, args: tuple, T: float, dt: float, dts: float, y0: np.ndarray,
                t0: np.ndarray, times: np.ndarray, **kwargs) -> np.ndarray:
 
@@ -397,15 +421,19 @@ class BaseBackend(CodeGen):
             # solve ivp via forward euler method (fixed integration step-size)
             results = self._solve_euler(func, args, T, dt, dts, y0, t0)
 
+        elif solver == 'heun':
+
+            # solve ivp via forward Heun's method (fixed integration step-size)
+            results = self._solve_heun(func, args, T, dt, dts, y0, t0)
+
+        elif solver == 'scipy':
+
+            results = self._solve_scipy(func, args, T, dt, y0, t0, times, **kwargs)
+
         else:
 
-            # solve ivp via scipy methods (solvers of various orders with adaptive step-size)
-            from scipy.integrate import solve_ivp
-            kwargs['t_eval'] = times
-
-            # call scipy solver
-            results = solve_ivp(fun=func, t_span=(t0, T), y0=y0, first_step=dt, args=args, **kwargs)
-            results = results['y'].T
+            raise PyRatesException('Invalid option for keyword `solver`. Please check the documentation of the '
+                                   '`CircuitTemplate.run` method for valid options.')
 
         return results
 
@@ -431,3 +459,36 @@ class BaseBackend(CodeGen):
             y += dt * rhs
 
         return state_rec
+
+    @staticmethod
+    def _solve_heun(func: Callable, args: tuple, T: float, dt: float, dts: float, y: np.ndarray, t0: np.ndarray):
+
+        # preparations for fixed step-size integration
+        idx = 0
+        steps = int(np.round(T / dt))
+        store_steps = int(np.round(T / dts))
+        store_step = int(np.round(dts / dt))
+        state_rec = np.zeros((store_steps, y.shape[0]) if y.shape else (store_steps, 1), dtype=y.dtype)
+
+        # solve ivp for forward Euler method
+        for step in range(t0, steps + t0):
+            if step % store_step == t0:
+                state_rec[idx, :] = y
+                idx += 1
+            rhs = func(step, y, *args)
+            y_0 = y + dt * rhs
+            y += dt/2 * (rhs + func(step, y_0, *args))
+
+        return state_rec
+
+    @staticmethod
+    def _solve_scipy(func: Callable, args: tuple, T: float, dt: float, y: np.ndarray, t0: np.ndarray, times: np.ndarray,
+                     **kwargs):
+
+        # solve ivp via scipy methods (solvers of various orders with adaptive step-size)
+        from scipy.integrate import solve_ivp
+        kwargs['t_eval'] = times
+
+        # call scipy solver
+        results = solve_ivp(fun=func, t_span=(t0, T), y0=y, first_step=dt, args=args, **kwargs)
+        return results['y'].T
