@@ -30,7 +30,7 @@
 """
 
 # external _imports
-from typing import Any, Callable, Union, Iterable, Optional
+from typing import Any, Callable, Union, Iterable, Optional, Dict
 from networkx import MultiDiGraph
 from sympy import Symbol, Expr, Function
 import numpy as np
@@ -244,6 +244,7 @@ class ComputeGraph(MultiDiGraph):
         self.var_updates = {'DEs': dict(), 'non-DEs': dict()}
         self._eq_nodes = []
         self._state_var_indices = dict()
+        self._state_var_hist = dict()
 
     @property
     def state_vars(self):
@@ -393,8 +394,24 @@ class ComputeGraph(MultiDiGraph):
         code_gen.code.clear()
 
         # generate function head
+        add_hist_calls = self._state_var_hist and code_gen.add_hist_arg
         func_args = code_gen.generate_func_head(func_name=func_name, state_var=state_var_key, return_var=rhs_var_key,
-                                                func_args=[self.get_var(arg) for arg in func_args])
+                                                func_args=[self.get_var(arg) for arg in func_args],
+                                                add_hist_func=add_hist_calls)
+
+        # extract state variable histories for delayed interactions
+        code_gen.add_linebreak()
+        for var, delays in self._state_var_hist.items():
+
+            # extract index of variable in state vector
+            idx = self._state_var_indices[var]
+
+            # extract state variable history from backend-specific buffer
+            if type(idx) is not int:
+                if idx[1]-idx[0] < 2:
+                    idx = idx[0]
+            for delay, v_hist in delays.items():  # type: ComputeVar, str
+                code_gen.add_var_hist(lhs=v_hist, delay=delay, state_idx=idx, var=var)
 
         # add lines from function body after function head
         code_gen.add_linebreak()
@@ -405,17 +422,33 @@ class ComputeGraph(MultiDiGraph):
         code_gen.generate_func_tail(rhs_var=rhs_var_key)
 
         # generate the function (and write to file, optionally)
-        func = code_gen.generate_func(func_name=func_name, to_file=to_file, func_args=func_args[3:],
+        func_args_tmp = func_args[4:] if add_hist_calls else func_args[3:]
+        func = code_gen.generate_func(func_name=func_name, to_file=to_file, func_args=func_args_tmp,
                                       state_vars=self.state_vars, **kwargs)
 
         # OPTIONAL: write function arguments (state vectors and constants) to file
         c_fn = kwargs.pop('constants_file_name', None)
         if c_fn:
-            arg_dict = {arg: self.get_var(arg).value for arg in func_args}
-            fn = f'{self.backend._fdir}/{c_fn}' if self.backend._fdir else c_fn
-            np.savez(fn, **arg_dict)
+            arg_dict = {arg: self.get_var(arg).value for arg in func_args if arg != 'hist'}
+            if code_gen.lags:
+                arg_dict['lags'] = list(code_gen.lags.keys())
+            fn = f'{self.backend.fdir}/{c_fn}' if self.backend.fdir else c_fn
+            code_gen.to_file(fn, **arg_dict)
 
-        return func, tuple([self.get_var(arg, from_backend=True) for arg in func_args])
+        # finalize the function arguments
+        fargs = []
+        for arg in func_args:
+            if arg == 'hist':
+                if 'hist' in kwargs:
+                    arg = kwargs.pop('hist')
+                else:
+                    y_init = np.asarray(state_vec[:])
+                    arg = code_gen.get_hist_func(y_init)
+                fargs.append(arg)
+            else:
+                fargs.append(self.get_var(arg, from_backend=True))
+
+        return func, tuple(fargs)
 
     def run(self, func: Callable, func_args: tuple, T: float, dt: float, dts: Optional[float] = None,
             outputs: Optional[dict] = None, **kwargs):
@@ -467,8 +500,6 @@ class ComputeGraph(MultiDiGraph):
             code_gen.add_var_update(lhs=self.get_var(var), rhs=f"y{rhs_idx}")
             rhs_indices_str.append(idx)
 
-        code_gen.add_linebreak()
-
         # get equation string and argument list for each non-DE node at the end of the compute graph hierarchy
         func_args2, delete_args1 = self._generate_update_equations(differential_equations=False)
         code_gen.add_linebreak()
@@ -616,18 +647,20 @@ class ComputeGraph(MultiDiGraph):
         # initializations
         index_args = []
         func = ""
-        var = str(expr.args[0]) if expr.args else ""
         idx = ""
 
         # ensure expression string exists
         if not expr_str:
             expr_str = str(expr)
 
-            # transform expression arguments into strings
-            for arg in expr.args:
-                expr_part, args, _, _ = self._expr_to_str(arg, **kwargs)
-                expr_str = expr_str.replace(str(arg), expr_part)
-                index_args.extend(args)
+        # transform expression arguments into strings
+        expr_args = []
+        for arg in expr.args:
+            expr_part, args, _, _ = self._expr_to_str(arg, **kwargs)
+            expr_str = expr_str.replace(str(arg), expr_part)
+            index_args.extend(args)
+            expr_args.append(expr_part)
+        var = str(expr_args[0]) if expr.args else ""
 
         # process indexing operations
         #############################
@@ -673,6 +706,16 @@ class ComputeGraph(MultiDiGraph):
             # replace `no_op` calls with first argument to the function call
             expr_str = self._process_func_call(expr=expr_str, func='identity', replacement=var)
 
+        if 'past(' in expr_str:
+
+            # replace past calls with the delayed version of the backend variable
+            try:
+                delay = self.get_var(expr.args[1].name)
+            except AttributeError:
+                delay = float(expr.args[1])
+            replacement = self._get_var_hist(var=var, delay=delay)
+            expr_str = self._process_func_call(expr=expr_str, func="past", replacement=replacement)
+
         # backend-specific function call adjustments
         expr_str = self.backend.expr_to_str(expr_str, expr.args)
 
@@ -791,6 +834,18 @@ class ComputeGraph(MultiDiGraph):
 
         return idx_str
 
+    def _get_var_hist(self, var: str, delay: Union[ComputeVar, float]):
+
+        if var not in self._state_var_hist:
+            self._state_var_hist[var] = dict()
+        if delay not in self._state_var_hist[var]:
+            var_hist = f'{var}_hist{len(self._state_var_hist[var])}'
+            self.add_var(var_hist, value=self.get_var(var).value, vtype='variable')
+            self._state_var_hist[var][delay] = var_hist
+        else:
+            var_hist = self._state_var_hist[var][delay]
+        return var_hist
+
     def _prune(self):
 
         # remove all subgraphs that contain constants only
