@@ -35,28 +35,5 @@
 
 # template-based interface
 from pyrates.frontend import template
-from pyrates.frontend.fileio import yaml
+from pyrates.frontend.fileio import yaml, pickle, save
 from pyrates.frontend.template import CircuitTemplate, NodeTemplate, EdgeTemplate, OperatorTemplate
-
-
-# The following function are shorthands that bridge multiple interface steps
-def circuit_from_yaml(path: str):
-    """Directly return CircuitIR instance from a yaml file."""
-    return CircuitTemplate.from_yaml(path).apply()
-
-
-def node_from_yaml(path: str):
-    """Directly return NodeIR instance from a yaml file."""
-    return NodeTemplate.from_yaml(path).apply()
-
-
-def edge_from_yaml(path: str):
-    """Directly return EdgeIR instance from a yaml file."""
-
-    return EdgeTemplate.from_yaml(path).apply()
-
-
-def operator_from_yaml(path: str):
-    """Directly return OperatorIR instance from a yaml file."""
-
-    return OperatorTemplate.from_yaml(path).apply()
