@@ -33,54 +33,12 @@ from typing import Iterator, Dict, List
 import numpy as _np
 from networkx import DiGraph, find_cycle, NetworkXNoCycle
 
-from pyrates import PyRatesException
+from pyrates.backend import PyRatesException
 from pyrates.ir.operator import OperatorIR
 
 __author__ = "Daniel Rose"
 __status__ = "Development"
 
-# define cache for OperatorGraph instances
-
-op_graph_cache = {}
-
-
-def cache_op_graph(cls):
-    """Cache unique instances of operator graphs and return the instance. If hash of Operator graph is not known yet,
-    a new instance will be created. Otherwise, an instance from cash will be returned."""
-
-    def cache_func(operators: Dict[str, OperatorIR] = None, template: str = ""):
-
-        if operators is None:
-            operators = {}
-
-        # compute hash from incoming operators. Different order of operators in input might lead to different hash.
-        h = hash(tuple(operators.values()))
-
-        changed_labels = False
-        try:
-            op_graph = op_graph_cache[h]
-        except KeyError:
-            op_graph = cls(operators, template)
-            # test, if hash computation leads to same result
-            assert h == hash(op_graph)
-            op_graph_cache[h] = op_graph
-        else:
-            # temporary workaround to obtain correct labels for values.
-            changed_labels = {}
-            for name, op in operators.items():
-                op_hash = hash(op)
-                for cached_name, cached_op in op_graph:
-                    if op_hash == hash(cached_op["operator"]):
-                        changed_labels[name] = cached_name
-                        # should only ever be able to find one, right? this fails (either way, if two operators with
-                        # identical hash exist in the same operator graph
-                        # fixme
-                        break
-
-        return op_graph, changed_labels
-
-    return cache_func
-
 
 class OperatorGraph(DiGraph):
     """Intermediate representation for nodes and edges."""
@@ -97,7 +55,6 @@ class OperatorGraph(DiGraph):
         # collect all information about output variables of operators
         #############################################################
         all_outputs = {}  # type: Dict[str, List[str]]
-        # op_inputs, op_outputs = set(), set()
         for key, operator in operators.items():
 
             # add operator as node to local operator_graph
@@ -196,18 +153,6 @@ class VectorizedOperatorGraph(DiGraph):
 
         if op_graph is None:
             pass
-        # elif len(op_graph) == 0:
-        #     self.add_node("identity_operator",
-        #                   inputs=dict(in_var=dict(source=set(),
-        #                                           reduce_dim=True)),
-        #                   equations=["out_var = in_var"],
-        #                   variables=dict(in_var=dict(dtype="float32",
-        #                                              vtype="state_var",
-        #                                              shape=(1,)),
-        #                                  out_var=dict(dtype="float32",
-        #                                               vtype="state_var",
-        #                                               shape=(1,))),
-        #                   output="out_var")
         else:
             for op_key, data in op_graph:
                 try:
@@ -226,8 +171,11 @@ class VectorizedOperatorGraph(DiGraph):
                 op_values = deepcopy(values[op_key])
                 op_vars = self.operators[op_key]["variables"]
                 for var_key, value in op_values.items():
-                    op_vars[var_key]["value"] = [value]
-                # self.operators[op_key]["variables"] = op_vars
+                    if op_vars[var_key]["vtype"] == "input_variable":
+                        op_vars[var_key]["value"] = value
+                        op_vars[var_key]["vtype"] = "input"
+                    else:
+                        op_vars[var_key]["value"] = value if type(value) is list else [value]
 
             self.add_edges_from(op_graph.edges)
 
